<?php

namespace Drupal\jsonapi_hypermedia\Plugin\jsonapi_hypermedia\LinkProvider;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\jsonapi\JsonApiResource\ResourceObject;

/**
 * Trait ResourceObjectEntityLoaderTrait
 *
 * @internal
 */
trait ResourceObjectEntityLoaderTrait {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Sets the entity type manager.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   */
  protected function setEntityTypeManager(EntityTypeManagerInterface $entity_type_manager) {
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * Gets the entity represented by the given resource object.
   *
   * @param \Drupal\jsonapi\JsonApiResource\ResourceObject $resource_object
   *   The resource object.
   *
   * @return \Drupal\Core\Entity\EntityInterface|null
   *   The represented entity or NULL if the entity does not exist.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function loadEntityFromResourceObject(ResourceObject $resource_object) {
    $resource_type = $resource_object->getResourceType();
    $entity_type = $this->entityTypeManager->getDefinition($resource_type->getEntityTypeId());
    $entities = $this->entityTypeManager->getStorage($entity_type->id())->loadByProperties([
      $entity_type->getKey('uuid') => $resource_object->getId(),
    ]);
    return reset($entities);
  }

}
