<?php

namespace Drupal\jsonapi\Normalizer\JsonapiHypermediaImpostor;

use Drupal\Core\Render\RenderContext;
use Drupal\Core\Render\RendererInterface;
use Drupal\jsonapi\JsonApiResource\LinkCollection;
use Drupal\jsonapi\Normalizer\LinkCollectionNormalizer;
use Drupal\jsonapi\Normalizer\Value\CacheableNormalization;
use Drupal\jsonapi_hypermedia\Plugin\LinkProviderManagerInterface;

/**
 * Class JsonApiHypermediaLinkCollectionNormalizer.
 *
 * @internal
 */
final class JsonApiHypermediaLinkCollectionNormalizer extends LinkCollectionNormalizer {

  /**
   * The link provider plugin manager service.
   *
   * @var \Drupal\jsonapi_hypermedia\Plugin\LinkProviderManagerInterface
   */
  protected $linkProviderManager;

  /**
   * The renderer service.
   *
   * @var \Drupal\Core\Render\RendererInterface
   */
  protected $renderer;

  /**
   * Sets the link provider manager service.
   *
   * @param \Drupal\jsonapi_hypermedia\Plugin\LinkProviderManagerInterface $link_provider_manager
   *   The link provider manager.
   */
  public function setLinkProviderManager(LinkProviderManagerInterface $link_provider_manager) {
    $this->linkProviderManager = $link_provider_manager;
  }

  /**
   * Set the renderer.
   *
   * @param \Drupal\Core\Render\RendererInterface $renderer
   *   The renderer service.
   */
  public function setRenderer(RendererInterface $renderer) {
    $this->renderer = $renderer;
  }

  /**
   * @inheritdoc
   */
  public function normalize($link_collection, $format = NULL, array $context = []) {
    assert($link_collection instanceof LinkCollection);
    $render_context = new RenderContext();
    $custom_links =$this->renderer->executeInRenderContext($render_context, function () use ($link_collection) {
      return $this->linkProviderManager->getLinkCollection($link_collection->getContext());
    });
    $normalized = [];
    /* @var \Drupal\jsonapi\JsonApiResource\Link $link */
    foreach (LinkCollection::merge($link_collection, $custom_links) as $key => $links) {
      $is_multiple = count($links) > 1;
      foreach ($links as $link) {
        $normalization = [];
        $link_key = $is_multiple ? sprintf('%s:%s', $key, $this->hashByHref($link)) : $key;
        $rel = $link->getLinkRelationTypes();
        $attributes = $link->getTargetAttributes();
        $normalization['href'] = $link->getHref();
        if (!empty($attributes)) {
          $normalization['meta']['linkParams'] = $this->serializer->normalize($attributes, $format, $context);
        }
        if (!empty($rel)) {
          // We have to call array_values() on $rel to reset the array's
          // numerical index. Otherwise, it can output arrays with missing
          // indices and json_encode will serialize these as objects. This is a
          // band-aid. The real fix probably needs to be made in Link::merge(),
          // Link::__construct() or LinkCollection::withLink() because one of
          // these methods is likely where the array's pointer is getting out of
          // place.
          $rel = array_values($rel);
          if (count($rel) > 1 || $rel[0] !== $link_key) {
            $normalization['meta']['linkParams']['rel'] = $rel;
          }
        }
        $normalized[$link_key] = new CacheableNormalization($link, $normalization);
      }
    }
    $normalized_links = CacheableNormalization::aggregate($normalized);
    return !$render_context->isEmpty()
      ? $normalized_links->withCacheableDependency($render_context->pop())
      : $normalized_links;
  }

}
